//-*-C++-*- Header file for the AudioDev Class
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Artistic License, included as the file
// "LICENSE" in the source code archive.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of the file "LICENSE", containing
// the License John Weiss originally placed this program under.
//
// This file is unreadable, because the comment-docifier is
// brainedead.  Try a "make srcdocs" to generate the
// HTML-documentation.
//
#ifndef _AudioDev_h_
#define _AudioDev_h_
#include <stdlib.h>
#include <time.h>
#include <qobject.h>
#include "Catalog.h"


/**
  The AudioDev class is the base class for all audio player devices.
  Its purpose is to provide something of a consistent template
  for other QObjects to make use of.
  
  Most importantly, it provides access to the audio device's
  catalog. 

  The documentation for each slot and signal describes how it should
  be implemented by a child class.  The slots themselves are all
  abstract functions.  So, you can't create a base {\bf AudioDev}
  object as a dummy argument for the \Ref{AudioPanel}.

  You should never try to reimplement the signals.  Just use them as
  described.  [If your child class implements any new signals, you'll
  need to connect them to a slot on your own.]

  {\bf
  \begin{verbatim}
  
  ----------------------------------------------------------
  ----------------------------------------------------------
  \end{verbatim}
  }

  @memo Base class for audio playing devices.
  @see Catalog, AudioCD

  */
class AudioDev : public QObject
{
	 //
	 Q_OBJECT

   public:
	 /**
	   Points to the \Ref{Catalog} associated with the current
	   device.  Any child class should set this pointer whenever the
	   \Ref{Catalog} gets created, destroyed, or recreated.
	   @memo
	   */
	 Catalog *ToC;

	 /**
	   Class Constructor.  Seeds the random number generator with
	   the current time and sets \Ref{ToC} to the null-pointer.
	   @memo
	   */
	 AudioDev( void ) {
	    srand48((long)time((time_t *)0));
	    ToC = (Catalog *)0;
	 }

	 /** Abstract function.  

	   Implement this method to initialize the audio device associated
	   with the file, {\em devname}, for the first time.  Should also
	   create a \Ref{Catalog} object to hold the ToC.

	   If your new device plays some sort of file, you probably won't
	   need {\em devname}.  Instead, you'll probably want to open
	   some sort of file dialog to get the filename.
	   @memo
	   */
	 virtual void DeviceFirstInit( char *devname ) = 0;

	 /** Abstract function.    

	   Implement this method to start a new program after changes to
	   the ToC.
	   @memo
	   */
	 virtual void newProgram(void) = 0;

	 /** Abstract function.    

	   Implement this method to emit all display-related signals.
	   Other classes use this to signal the {\bf AudioDev} after any
	   major change.
	   @memo
	   */
	 virtual void cleanDisplay(void) = 0;

	 /** Abstract function.    

	   Implement this method to update track information from the
	   device. 
	   @memo
	   */
	 virtual void updateToCTrkInf(void) = 0;

	 //
   public slots:
	 /**
	   {\em Public Slot} {\tt [virtual]}
	   
	   Abstract function.    

	   Implement this method to initiate a forward scan.

	   This slot typically receives a {\tt pressed} signal from a
	   "scan-forward" button.
	   
	   @memo [{\tt virtual public slot}]
	   */
	 virtual void StartScanFwd() = 0;

	 /**
	   {\em Public Slot} {\tt [virtual]}
	   
	   Abstract function.    

	   Implement this method to initiate a reverse scan.

	   This slot typically receives a {\tt pressed} signal from a
	   "scan-backward" button.
	   
	   @memo [{\tt virtual public slot}]
	   */
	 virtual void StartScanRev() = 0;

	 /**
	   {\em Public Slot} {\tt [virtual]}
	   
	   Abstract function.    

	   Implement this method to complete a scan operation.  

	   Typically, this slot will receive the {\tt released} signal from
	   a scan-forward or scan-backward button.
	   
	   @memo [{\tt virtual public slot}]
	   @see StartScanFwd, StartScanRev
	   */
	 virtual void StopScan() = 0;

	 /**
	   {\em Public Slot} {\tt [virtual]}
	   
	   Abstract function.    

	   Implement this method to control playing and pausing.  It
	   should start the device playing if it's stopped, pause play if
	   it's playing, or resume play if it's paused.
	   Your implementation should handle all of this internally.
	   
	   @memo [{\tt virtual public slot}] */
	 virtual void StartPlayOrPause() = 0;

	 /**
	   {\em Public Slot} {\tt [virtual]}
	   
	   Abstract function.    

	   Implement this method to stop play.
	   
	   @memo [{\tt virtual public slot}]
	   */
	 virtual void StopPlay() = 0;

	 /**
	   {\em Public Slot} {\tt [virtual]}
	   
	   Abstract function.    

	   Implement this method to move to the next item in the playlist.
	   
	   @memo [{\tt virtual public slot}]
	   */
	 virtual void Next() = 0;

	 /**
	   {\em Public Slot} {\tt [virtual]}
	   
	   Abstract function.    

	   Implement this method to move to the previous item in the
	   playlist.
	   
	   @memo [{\tt virtual public slot}] */
	 virtual void Prev() = 0;

	 /**
	   {\em Public Slot} {\tt [virtual]}
	   
	   Abstract function.    

	   Implement this method to increase the volume by a
	   pre-programmed step.
	   
	   @memo [{\tt virtual public slot}] */
	 virtual void IncVol() = 0;

	 /**
	   {\em Public Slot} {\tt [virtual]}
	   
	   Abstract function.    

	   Implement this method to decrease the volume by a
	   pre-programmed step.
	   
	   @memo [{\tt virtual public slot}] */
	 virtual void DecVol() = 0;

	 /**
	   {\em Public Slot} {\tt [virtual]}
	   
	   Abstract function.    

	   Implement this method to emit signals to change the time mode
	   display icon on the \Ref{AudioPanel}.  The parameter will be a
	   \Ref{TimeMode} {\bf enum}.

	   You might also use this method to alter any internal flags
	   that control how the time gets displayed.
	   
	   @see newTime, TimeMode
	   @memo [{\tt virtual public slot}] */
	 virtual void newTimeMode(int) = 0;

	 /**
	   {\em Public Slot} {\tt [virtual]}
	   
	   Abstract function.    

	   Implement this method to emit signals to change the play mode
	   display icon on the \Ref{AudioPanel}.  The parameter will be a
	   \Ref{PlayState} {\bf enum} OR-ed with a \Ref{PlayMode} {\bf
	   enum}.

	   This method might also perform any necessary operations to
	   alter the time mode.
	   
	   @memo [{\tt virtual public slot}] */
	 virtual void newPlayMode(int) = 0;

	 /**
	   {\em Public Slot} {\tt [virtual]}
	   
	   Abstract function.    

	   Implement this method to eject/retract a CD, or load a new set
	   of audio files.  You'll need to code this one appropriately.
	   
	   @memo [{\tt virtual public slot}]
	   */
	 virtual void LoadOrEject() = 0;

	 //
   signals:
	 /**
	   {\em Public Signal} {\tt [virtual]}
	   
	   Use to emit the new volume after a change.
	   
	   @memo [{\tt virtual signal}]
	   */
	 virtual void newVolume(float);

	 /**
	   {\em Public Signal} {\tt [virtual]}
	   
	   Use to emit a time to be displayed.  This can be the elapsed
	   time, the remaining time, the total remaining time, etc...
	   
	   @memo [{\tt virtual signal}] */
	 virtual void newTime(int);

	 /**
	   {\em Public Signal} {\tt [virtual]}
	   
	   Emit this when the current item changes, either automatically
	   or through user intervention.  Emits the item number.
	   
	   @memo [{\tt virtual signal}] */
	 virtual void newItemNumber(int);

	 /**
	   {\em Public Signal} {\tt [virtual]}
	   
	   Emit this when the current item changes, either automatically
	   or through user intervention.  Emits the label in \Ref{ToC}
	   corresponding to the current item.
	   
	   @memo [{\tt virtual signal}] */
	 virtual void newItemLabel(char *);

	 /**
	   {\em Public Signal} {\tt [virtual]}
	   
	   Use to emit the program title specified in \Ref{ToC}.
	   
	   @memo [{\tt virtual signal}] */
	 virtual void newProgLabel(char *);

	 /**
	   {\em Public Signal} {\tt [virtual]}
	   
	   Use to emit the program's artist, as specified by \Ref{ToC}.
	   
	   @memo [{\tt virtual signal}] */
	 virtual void newProgArtist(char *);

	 /**
	   {\em Public Signal} {\tt [virtual]}
	   
	   Emit this when the play state changed.  {\em E.g.:} the device
	   finishes playing the last item in the program and
	   automatically stops.
	   
	   The value it emits should be a \Ref{PlayState} {\bf enum}
	   OR-ed with a \Ref{PlayMode} {\bf enum} containing the current
	   play mode.
	   @memo [{\tt virtual signal}] */
	 virtual void newPlayState(int);

	 /**
	   {\em Public Signal} {\tt [virtual]}
	   
	   Emit this when the device wants to check its \Ref{ToC} to see
	   if it's in any of the databases.
	   
	   @see MaintainDB
	   @memo [{\tt virtual signal}] */
	 virtual void checkProgramCatalog();
};


#endif // _AudioDev_h_
//||||||||||||||||||||||||||||||||
//
// End
// 
